
exports.up = async function(knex, Promise) {
    await knex.schema.dropTable('posts');
    await knex.schema.table('messages', function(table){    
        table.dropColumn('authors');
    });
    await knex.schema.createTable('private_post', function(table){
        table.inherits('messages');
        table.string('author_uid', 128).notNullable();
        table.enum('author_db', ['ldap']).notNullable().defaultTo('ldap');
        table.string('recipient_uid', 128).notNullable();
    });
    await knex.schema.createTable('question', function(table){
        table.inherits('messages');
        table.string('author_uid', 128).notNullable();
        table.enum('author_db', ['ldap']).notNullable().defaultTo('ldap');
        table.string('recipient_uid', 128).notNullable();
    });
    await knex.schema.createTable('answer', function(table){
        table.inherits('messages');
        table.string('author_uid', 128).notNullable();
        table.string('recipient_uid', 128).notNullable();
        table.string('for_question', 128).notNullable();
    });
    await knex.schema.table('events', function(table){
        table.specificType('authors', knex.raw('varchar(128)[]'));
    });
    await knex.schema.table('announcements', function(table){
        table.specificType('authors', knex.raw('varchar(128)[]'));
    });
    return;
};

exports.down = async function(knex, Promise) {
    await knex.schema.dropTable('answer');
    await knex.schema.dropTable('question');
    await knex.schema.dropTable('private_post');
    await knex.schema.table('messages', function(table){
        table.specificType('authors', knex.raw('varchar(128)[]'));
    });
    await knex.schema.createTable('posts', function(table) {
        table.inherits('messages');
    });
    return;
};
